@@ -28,7 +28,8 @@ import numpy as np
 from os import remove, path
 
 from tpot import TPOTClassifier, TPOTRegressor
-from tpot.export_utils import export_pipeline, generate_import_code, _indent, generate_pipeline_code, get_by_name
+from tpot.export_utils import export_pipeline, generate_import_code, _indent, \
+    generate_pipeline_code, get_by_name, set_param_recursive
 from tpot.operator_utils import TPOTOperatorClassFactory
 from tpot.config.classifier import classifier_config_dict
 
@@ -36,8 +37,8 @@ from sklearn.datasets import load_digits
 from sklearn.model_selection import train_test_split
 from deap import creator
 
-from nose.tools import assert_raises, assert_equal
-
+from nose.tools import assert_raises, assert_equal, nottest
+train_test_split = nottest(train_test_split)
 test_operator_key_1 = 'sklearn.feature_selection.SelectPercentile'
 test_operator_key_2 = 'sklearn.feature_selection.SelectFromModel'
 TPOTSelectPercentile, TPOTSelectPercentile_args = TPOTOperatorClassFactory(
@@ -50,43 +51,48 @@ TPOTSelectFromModel, TPOTSelectFromModel_args = TPOTOperatorClassFactory(
     classifier_config_dict[test_operator_key_2]
 )
 
-mnist_data = load_digits()
+digits_data = load_digits()
 training_features, testing_features, training_target, testing_target = \
-    train_test_split(mnist_data.data.astype(np.float64), mnist_data.target.astype(np.float64), random_state=42)
+    train_test_split(digits_data.data.astype(np.float64), digits_data.target.astype(np.float64), random_state=42)
 
+tpot_obj = TPOTClassifier()
+tpot_obj._fit_init()
+
+tpot_obj_reg = TPOTRegressor()
+tpot_obj_reg._fit_init()
 
 def test_export_random_ind():
     """Assert that the TPOTClassifier can generate the same pipeline export with random seed of 39."""
-    tpot_obj = TPOTClassifier(random_state=39)
+    tpot_obj = TPOTClassifier(random_state=39, config_dict="TPOT light")
+    tpot_obj._fit_init()
     tpot_obj._pbar = tqdm(total=1, disable=True)
     pipeline = tpot_obj._toolbox.individual()
     expected_code = """import numpy as np
 import pandas as pd
-from sklearn.feature_selection import SelectPercentile, f_classif
 from sklearn.model_selection import train_test_split
-from sklearn.pipeline import make_pipeline
-from sklearn.tree import DecisionTreeClassifier
+from sklearn.naive_bayes import BernoulliNB
+from tpot.export_utils import set_param_recursive
 
-# NOTE: Make sure that the class is labeled 'target' in the data file
+# NOTE: Make sure that the outcome column is labeled 'target' in the data file
 tpot_data = pd.read_csv('PATH/TO/DATA/FILE', sep='COLUMN_SEPARATOR', dtype=np.float64)
-features = tpot_data.drop('target', axis=1).values
+features = tpot_data.drop('target', axis=1)
 training_features, testing_features, training_target, testing_target = \\
-            train_test_split(features, tpot_data['target'].values, random_state=42)
+            train_test_split(features, tpot_data['target'], random_state=39)
 
-exported_pipeline = make_pipeline(
-    SelectPercentile(score_func=f_classif, percentile=65),
-    DecisionTreeClassifier(criterion="gini", max_depth=7, min_samples_leaf=4, min_samples_split=18)
-)
+exported_pipeline = BernoulliNB(alpha=1.0, fit_prior=False)
+# Fix random state for all the steps in exported pipeline
+set_param_recursive(exported_pipeline.steps, 'random_state', 39)
 
 exported_pipeline.fit(training_features, training_target)
 results = exported_pipeline.predict(testing_features)
 """
-    assert expected_code == export_pipeline(pipeline, tpot_obj.operators, tpot_obj._pset)
+    exported_code = export_pipeline(pipeline, tpot_obj.operators, tpot_obj._pset, random_state=tpot_obj.random_state)
+
+    assert expected_code == exported_code
 
 
 def test_export():
     """Assert that TPOT's export function throws a RuntimeError when no optimized pipeline exists."""
-    tpot_obj = TPOTClassifier()
     assert_raises(RuntimeError, tpot_obj.export, "test_export.py")
     pipeline_string = (
         'KNeighborsClassifier(CombineDFs('
@@ -104,9 +110,42 @@ def test_export():
     remove("test_export.py") # clean up exported file
 
 
+def test_export_2():
+    """Assert that TPOT's export function returns the expected pipeline text as a string."""
+
+    pipeline_string = (
+        'KNeighborsClassifier('
+        'input_matrix, '
+        'KNeighborsClassifier__n_neighbors=10, '
+        'KNeighborsClassifier__p=1, '
+        'KNeighborsClassifier__weights=uniform'
+        ')'
+    )
+    pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
+    tpot_obj._optimized_pipeline = pipeline
+    expected_code = """import numpy as np
+import pandas as pd
+from sklearn.model_selection import train_test_split
+from sklearn.neighbors import KNeighborsClassifier
+
+# NOTE: Make sure that the outcome column is labeled 'target' in the data file
+tpot_data = pd.read_csv('PATH/TO/DATA/FILE', sep='COLUMN_SEPARATOR', dtype=np.float64)
+features = tpot_data.drop('target', axis=1)
+training_features, testing_features, training_target, testing_target = \\
+            train_test_split(features, tpot_data['target'], random_state=None)
+
+exported_pipeline = KNeighborsClassifier(n_neighbors=10, p=1, weights="uniform")
+
+exported_pipeline.fit(training_features, training_target)
+results = exported_pipeline.predict(testing_features)
+"""
+    assert expected_code == tpot_obj.export()
+
+
 def test_generate_pipeline_code():
     """Assert that generate_pipeline_code() returns the correct code given a specific pipeline."""
-    tpot_obj = TPOTClassifier()
+
+    tpot_obj._fit_init()
     pipeline = [
         'KNeighborsClassifier',
         [
@@ -148,7 +187,7 @@ def test_generate_pipeline_code():
 
 def test_generate_pipeline_code_2():
     """Assert that generate_pipeline_code() returns the correct code given a specific pipeline with two CombineDFs."""
-    tpot_obj = TPOTClassifier()
+
     pipeline = [
         'KNeighborsClassifier',
         [
@@ -200,7 +239,7 @@ def test_generate_pipeline_code_2():
 
 def test_generate_import_code():
     """Assert that generate_import_code() returns the correct set of dependancies for a given pipeline."""
-    tpot_obj = TPOTClassifier()
+
     pipeline = creator.Individual.from_string('GaussianNB(RobustScaler(input_matrix))', tpot_obj._pset)
 
     expected_code = """import numpy as np
@@ -215,7 +254,7 @@ from sklearn.preprocessing import RobustScaler
 
 def test_generate_import_code_2():
     """Assert that generate_import_code() returns the correct set of dependancies and dependancies are importable."""
-    tpot_obj = TPOTClassifier()
+
     pipeline_string = (
         'KNeighborsClassifier(CombineDFs('
         'DecisionTreeClassifier(input_matrix, DecisionTreeClassifier__criterion=gini, '
@@ -241,7 +280,6 @@ from tpot.builtins import StackingEstimator, ZeroCount
 
 def test_operators():
     """Assert that the TPOT operators match the output of their sklearn counterparts."""
-    tpot_obj = TPOTClassifier(random_state=42)
     for op in tpot_obj.operators:
         check_export.description = ("Assert that the TPOT {} operator exports "
                                     "as expected".format(op.__name__))
@@ -263,7 +301,7 @@ def check_export(op, tpot_obj):
 
 def test_export_pipeline():
     """Assert that exported_pipeline() generated a compile source file as expected given a fixed pipeline."""
-    tpot_obj = TPOTClassifier()
+
     pipeline_string = (
         'KNeighborsClassifier(CombineDFs('
         'DecisionTreeClassifier(input_matrix, DecisionTreeClassifier__criterion=gini, '
@@ -283,11 +321,11 @@ from sklearn.pipeline import make_pipeline, make_union
 from sklearn.tree import DecisionTreeClassifier
 from tpot.builtins import StackingEstimator
 
-# NOTE: Make sure that the class is labeled 'target' in the data file
+# NOTE: Make sure that the outcome column is labeled 'target' in the data file
 tpot_data = pd.read_csv('PATH/TO/DATA/FILE', sep='COLUMN_SEPARATOR', dtype=np.float64)
-features = tpot_data.drop('target', axis=1).values
+features = tpot_data.drop('target', axis=1)
 training_features, testing_features, training_target, testing_target = \\
-            train_test_split(features, tpot_data['target'].values, random_state=42)
+            train_test_split(features, tpot_data['target'], random_state=None)
 
 exported_pipeline = make_pipeline(
     make_union(
@@ -305,7 +343,7 @@ results = exported_pipeline.predict(testing_features)
 
 def test_export_pipeline_2():
     """Assert that exported_pipeline() generated a compile source file as expected given a fixed simple pipeline (only one classifier)."""
-    tpot_obj = TPOTClassifier()
+
     pipeline_string = (
         'KNeighborsClassifier('
         'input_matrix, '
@@ -320,11 +358,11 @@ import pandas as pd
 from sklearn.model_selection import train_test_split
 from sklearn.neighbors import KNeighborsClassifier
 
-# NOTE: Make sure that the class is labeled 'target' in the data file
+# NOTE: Make sure that the outcome column is labeled 'target' in the data file
 tpot_data = pd.read_csv('PATH/TO/DATA/FILE', sep='COLUMN_SEPARATOR', dtype=np.float64)
-features = tpot_data.drop('target', axis=1).values
+features = tpot_data.drop('target', axis=1)
 training_features, testing_features, training_target, testing_target = \\
-            train_test_split(features, tpot_data['target'].values, random_state=42)
+            train_test_split(features, tpot_data['target'], random_state=None)
 
 exported_pipeline = KNeighborsClassifier(n_neighbors=10, p=1, weights="uniform")
 
@@ -336,7 +374,7 @@ results = exported_pipeline.predict(testing_features)
 
 def test_export_pipeline_3():
     """Assert that exported_pipeline() generated a compile source file as expected given a fixed simple pipeline with a preprocessor."""
-    tpot_obj = TPOTClassifier()
+
     pipeline_string = (
         'DecisionTreeClassifier(SelectPercentile(input_matrix, SelectPercentile__percentile=20),'
         'DecisionTreeClassifier__criterion=gini, DecisionTreeClassifier__max_depth=8,'
@@ -351,11 +389,11 @@ from sklearn.model_selection import train_test_split
 from sklearn.pipeline import make_pipeline
 from sklearn.tree import DecisionTreeClassifier
 
-# NOTE: Make sure that the class is labeled 'target' in the data file
+# NOTE: Make sure that the outcome column is labeled 'target' in the data file
 tpot_data = pd.read_csv('PATH/TO/DATA/FILE', sep='COLUMN_SEPARATOR', dtype=np.float64)
-features = tpot_data.drop('target', axis=1).values
+features = tpot_data.drop('target', axis=1)
 training_features, testing_features, training_target, testing_target = \\
-            train_test_split(features, tpot_data['target'].values, random_state=42)
+            train_test_split(features, tpot_data['target'], random_state=None)
 
 exported_pipeline = make_pipeline(
     SelectPercentile(score_func=f_classif, percentile=20),
@@ -370,7 +408,7 @@ results = exported_pipeline.predict(testing_features)
 
 def test_export_pipeline_4():
     """Assert that exported_pipeline() generated a compile source file as expected given a fixed simple pipeline with input_matrix in CombineDFs."""
-    tpot_obj = TPOTClassifier()
+
     pipeline_string = (
         'KNeighborsClassifier(CombineDFs('
         'DecisionTreeClassifier(input_matrix, DecisionTreeClassifier__criterion=gini, '
@@ -391,11 +429,11 @@ from tpot.builtins import StackingEstimator
 from sklearn.preprocessing import FunctionTransformer
 from copy import copy
 
-# NOTE: Make sure that the class is labeled 'target' in the data file
+# NOTE: Make sure that the outcome column is labeled 'target' in the data file
 tpot_data = pd.read_csv('PATH/TO/DATA/FILE', sep='COLUMN_SEPARATOR', dtype=np.float64)
-features = tpot_data.drop('target', axis=1).values
+features = tpot_data.drop('target', axis=1)
 training_features, testing_features, training_target, testing_target = \\
-            train_test_split(features, tpot_data['target'].values, random_state=42)
+            train_test_split(features, tpot_data['target'], random_state=None)
 
 exported_pipeline = make_pipeline(
     make_union(
@@ -413,14 +451,13 @@ results = exported_pipeline.predict(testing_features)
 
 def test_export_pipeline_5():
     """Assert that exported_pipeline() generated a compile source file as expected given a fixed simple pipeline with SelectFromModel."""
-    tpot_obj = TPOTRegressor()
     pipeline_string = (
         'DecisionTreeRegressor(SelectFromModel(input_matrix, '
         'SelectFromModel__ExtraTreesRegressor__max_features=0.05, SelectFromModel__ExtraTreesRegressor__n_estimators=100, '
         'SelectFromModel__threshold=0.05), DecisionTreeRegressor__max_depth=8,'
         'DecisionTreeRegressor__min_samples_leaf=5, DecisionTreeRegressor__min_samples_split=5)'
     )
-    pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
+    pipeline = creator.Individual.from_string(pipeline_string, tpot_obj_reg._pset)
     expected_code = """import numpy as np
 import pandas as pd
 from sklearn.ensemble import ExtraTreesRegressor
@@ -429,11 +466,11 @@ from sklearn.model_selection import train_test_split
 from sklearn.pipeline import make_pipeline
 from sklearn.tree import DecisionTreeRegressor
 
-# NOTE: Make sure that the class is labeled 'target' in the data file
+# NOTE: Make sure that the outcome column is labeled 'target' in the data file
 tpot_data = pd.read_csv('PATH/TO/DATA/FILE', sep='COLUMN_SEPARATOR', dtype=np.float64)
-features = tpot_data.drop('target', axis=1).values
+features = tpot_data.drop('target', axis=1)
 training_features, testing_features, training_target, testing_target = \\
-            train_test_split(features, tpot_data['target'].values, random_state=42)
+            train_test_split(features, tpot_data['target'], random_state=None)
 
 exported_pipeline = make_pipeline(
     SelectFromModel(estimator=ExtraTreesRegressor(max_features=0.05, n_estimators=100), threshold=0.05),
@@ -443,7 +480,45 @@ exported_pipeline = make_pipeline(
 exported_pipeline.fit(training_features, training_target)
 results = exported_pipeline.predict(testing_features)
 """
-    assert expected_code == export_pipeline(pipeline, tpot_obj.operators, tpot_obj._pset)
+    assert expected_code == export_pipeline(pipeline, tpot_obj_reg.operators, tpot_obj_reg._pset)
+
+
+def test_export_pipeline_6():
+    """Assert that exported_pipeline() generated a compile source file with random_state and data_file_path."""
+
+    pipeline_string = (
+        'KNeighborsClassifier('
+        'input_matrix, '
+        'KNeighborsClassifier__n_neighbors=10, '
+        'KNeighborsClassifier__p=1, '
+        'KNeighborsClassifier__weights=uniform'
+        ')'
+    )
+    pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
+    expected_code = """import numpy as np
+import pandas as pd
+from sklearn.model_selection import train_test_split
+from sklearn.neighbors import KNeighborsClassifier
+from tpot.export_utils import set_param_recursive
+
+# NOTE: Make sure that the outcome column is labeled 'target' in the data file
+tpot_data = pd.read_csv('test_path', sep='COLUMN_SEPARATOR', dtype=np.float64)
+features = tpot_data.drop('target', axis=1)
+training_features, testing_features, training_target, testing_target = \\
+            train_test_split(features, tpot_data['target'], random_state=42)
+
+exported_pipeline = KNeighborsClassifier(n_neighbors=10, p=1, weights="uniform")
+# Fix random state for all the steps in exported pipeline
+set_param_recursive(exported_pipeline.steps, 'random_state', 42)
+
+exported_pipeline.fit(training_features, training_target)
+results = exported_pipeline.predict(testing_features)
+"""
+    exported_code = export_pipeline(pipeline, tpot_obj.operators,
+                                    tpot_obj._pset, random_state=42,
+                                    data_file_path='test_path')
+
+    assert expected_code == exported_code
 
 
 def test_operator_export():
@@ -465,14 +540,14 @@ def test_operator_export_2():
 
 def test_get_by_name():
     """Assert that the Operator class returns operators by name appropriately."""
-    tpot_obj = TPOTClassifier()
+
     assert get_by_name("SelectPercentile", tpot_obj.operators).__class__ == TPOTSelectPercentile.__class__
     assert get_by_name("SelectFromModel", tpot_obj.operators).__class__ == TPOTSelectFromModel.__class__
 
 
 def test_get_by_name_2():
     """Assert that get_by_name raises TypeError with a incorrect operator name."""
-    tpot_obj = TPOTClassifier()
+
     assert_raises(TypeError, get_by_name, "RandomForestRegressor", tpot_obj.operators)
     # use correct name
     ret_op_class = get_by_name("RandomForestClassifier", tpot_obj.operators)
@@ -480,7 +555,7 @@ def test_get_by_name_2():
 
 def test_get_by_name_3():
     """Assert that get_by_name raises ValueError with duplicate operators in operator dictionary."""
-    tpot_obj = TPOTClassifier()
+
     # no duplicate
     ret_op_class = get_by_name("SelectPercentile", tpot_obj.operators)
     # add a copy of TPOTSelectPercentile into operator list
@@ -505,9 +580,15 @@ test3"""
 
 def test_pipeline_score_save():
     """Assert that the TPOTClassifier can generate a scored pipeline export correctly."""
-    tpot_obj = TPOTClassifier(random_state=39)
+    tpot_obj = TPOTClassifier()
+    tpot_obj._fit_init()
     tpot_obj._pbar = tqdm(total=1, disable=True)
-    pipeline = tpot_obj._toolbox.individual()
+    pipeline_string = (
+        'DecisionTreeClassifier(SelectPercentile(input_matrix, SelectPercentile__percentile=20),'
+        'DecisionTreeClassifier__criterion=gini, DecisionTreeClassifier__max_depth=8,'
+        'DecisionTreeClassifier__min_samples_leaf=5, DecisionTreeClassifier__min_samples_split=5)'
+    )
+    pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
     expected_code = """import numpy as np
 import pandas as pd
 from sklearn.feature_selection import SelectPercentile, f_classif
@@ -515,22 +596,21 @@ from sklearn.model_selection import train_test_split
 from sklearn.pipeline import make_pipeline
 from sklearn.tree import DecisionTreeClassifier
 
-# NOTE: Make sure that the class is labeled 'target' in the data file
+# NOTE: Make sure that the outcome column is labeled 'target' in the data file
 tpot_data = pd.read_csv('PATH/TO/DATA/FILE', sep='COLUMN_SEPARATOR', dtype=np.float64)
-features = tpot_data.drop('target', axis=1).values
+features = tpot_data.drop('target', axis=1)
 training_features, testing_features, training_target, testing_target = \\
-            train_test_split(features, tpot_data['target'].values, random_state=42)
+            train_test_split(features, tpot_data['target'], random_state=None)
 
-# Score on the training set was:0.929813743
+# Average CV score on the training set was: 0.929813743
 exported_pipeline = make_pipeline(
-    SelectPercentile(score_func=f_classif, percentile=65),
-    DecisionTreeClassifier(criterion="gini", max_depth=7, min_samples_leaf=4, min_samples_split=18)
+    SelectPercentile(score_func=f_classif, percentile=20),
+    DecisionTreeClassifier(criterion="gini", max_depth=8, min_samples_leaf=5, min_samples_split=5)
 )
 
 exported_pipeline.fit(training_features, training_target)
 results = exported_pipeline.predict(testing_features)
 """
-
     assert_equal(expected_code, export_pipeline(pipeline, tpot_obj.operators, tpot_obj._pset, pipeline_score=0.929813743))
 
 
@@ -565,15 +645,15 @@ def test_imputer_in_export():
 import pandas as pd
 from sklearn.model_selection import train_test_split
 from sklearn.neighbors import KNeighborsClassifier
-from sklearn.preprocessing import Imputer
+from sklearn.impute import SimpleImputer
 
-# NOTE: Make sure that the class is labeled 'target' in the data file
+# NOTE: Make sure that the outcome column is labeled 'target' in the data file
 tpot_data = pd.read_csv('PATH/TO/DATA/FILE', sep='COLUMN_SEPARATOR', dtype=np.float64)
-features = tpot_data.drop('target', axis=1).values
+features = tpot_data.drop('target', axis=1)
 training_features, testing_features, training_target, testing_target = \\
-            train_test_split(features, tpot_data['target'].values, random_state=42)
+            train_test_split(features, tpot_data['target'], random_state=None)
 
-imputer = Imputer(strategy="median")
+imputer = SimpleImputer(strategy="median")
 imputer.fit(training_features)
 training_features = imputer.transform(training_features)
 testing_features = imputer.transform(testing_features)
@@ -585,3 +665,62 @@ results = exported_pipeline.predict(testing_features)
 """
 
     assert_equal(export_code, expected_code)
+
+
+def test_set_param_recursive():
+    tpot_obj = TPOTClassifier()
+    tpot_obj._fit_init()
+    """Assert that _set_param_recursive sets \"random_state\" to 42 in all steps in a simple pipeline."""
+    pipeline_string = (
+        'DecisionTreeClassifier(PCA(input_matrix, PCA__iterated_power=5, PCA__svd_solver=randomized), '
+        'DecisionTreeClassifier__criterion=gini, DecisionTreeClassifier__max_depth=8, '
+        'DecisionTreeClassifier__min_samples_leaf=5, DecisionTreeClassifier__min_samples_split=5)'
+    )
+
+    deap_pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
+    sklearn_pipeline = tpot_obj._toolbox.compile(expr=deap_pipeline)
+    set_param_recursive(sklearn_pipeline.steps, 'random_state', 42)
+    # assert "random_state" of PCA at step 1
+    assert getattr(sklearn_pipeline.steps[0][1], 'random_state') == 42
+    # assert "random_state" of DecisionTreeClassifier at step 2
+    assert getattr(sklearn_pipeline.steps[1][1], 'random_state') == 42
+
+
+def test_set_param_recursive_2():
+    """Assert that set_param_recursive sets \"random_state\" to 42 in nested estimator in SelectFromModel."""
+    pipeline_string = (
+        'DecisionTreeRegressor(SelectFromModel(input_matrix, '
+        'SelectFromModel__ExtraTreesRegressor__max_features=0.05, SelectFromModel__ExtraTreesRegressor__n_estimators=100, '
+        'SelectFromModel__threshold=0.05), DecisionTreeRegressor__max_depth=8,'
+        'DecisionTreeRegressor__min_samples_leaf=5, DecisionTreeRegressor__min_samples_split=5)'
+    )
+    tpot_obj = TPOTRegressor()
+    tpot_obj._fit_init()
+    deap_pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
+    sklearn_pipeline = tpot_obj._toolbox.compile(expr=deap_pipeline)
+    set_param_recursive(sklearn_pipeline.steps, 'random_state', 42)
+
+    assert getattr(getattr(sklearn_pipeline.steps[0][1], 'estimator'), 'random_state') == 42
+    assert getattr(sklearn_pipeline.steps[1][1], 'random_state') == 42
+
+
+def test_set_param_recursive_3():
+    """Assert that set_param_recursive sets \"random_state\" to 42 in nested estimator in StackingEstimator in a complex pipeline."""
+    pipeline_string = (
+        'DecisionTreeClassifier(CombineDFs('
+        'DecisionTreeClassifier(input_matrix, DecisionTreeClassifier__criterion=gini, '
+        'DecisionTreeClassifier__max_depth=8, DecisionTreeClassifier__min_samples_leaf=5,'
+        'DecisionTreeClassifier__min_samples_split=5),input_matrix) '
+        'DecisionTreeClassifier__criterion=gini, DecisionTreeClassifier__max_depth=8, '
+        'DecisionTreeClassifier__min_samples_leaf=5, DecisionTreeClassifier__min_samples_split=5)'
+    )
+    tpot_obj = TPOTClassifier()
+    tpot_obj._fit_init()
+
+    deap_pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
+    sklearn_pipeline = tpot_obj._toolbox.compile(expr=deap_pipeline)
+    set_param_recursive(sklearn_pipeline.steps, 'random_state', 42)
+
+    # StackingEstimator under the transformer_list of FeatureUnion
+    assert getattr(getattr(sklearn_pipeline.steps[0][1].transformer_list[0][1], 'estimator'), 'random_state') == 42
+    assert getattr(sklearn_pipeline.steps[1][1], 'random_state') == 42
