@@ -51,7 +51,11 @@ def get_by_name(opname, operators):
     return ret_op_class
 
 
-def export_pipeline(exported_pipeline, operators, pset, impute=False, pipeline_score=None, random_state=None):
+def export_pipeline(exported_pipeline,
+                    operators, pset,
+                    impute=False, pipeline_score=None,
+                    random_state=None,
+                    data_file_path=''):
     """Generate source code for a TPOT Pipeline.
 
     Parameters
@@ -62,6 +66,13 @@ def export_pipeline(exported_pipeline, operators, pset, impute=False, pipeline_s
         List of operator classes from operator library
     pipeline_score:
         Optional pipeline score to be saved to the exported file
+    impute: bool (False):
+        If impute = True, then adda a imputation step.
+    random_state: integer
+        Random seed in train_test_split function and exported pipeline.
+    data_file_path: string (default: '')
+        By default, the path of input dataset is 'PATH/TO/DATA/FILE' by default.
+        If data_file_path is another string, the path will be replaced.
 
     Returns
     -------
@@ -73,34 +84,36 @@ def export_pipeline(exported_pipeline, operators, pset, impute=False, pipeline_s
     pipeline_tree = expr_to_tree(exported_pipeline, pset)
 
     # Have the exported code import all of the necessary modules and functions
-    pipeline_text = generate_import_code(exported_pipeline, operators, impute)
+    pipeline_text = generate_import_code(exported_pipeline, operators, impute, random_state)
 
-    pipeline_code = pipeline_code_wrapper(generate_export_pipeline_code(pipeline_tree, operators))
+    pipeline_code = pipeline_code_wrapper(generate_export_pipeline_code(pipeline_tree, operators), random_state)
 
     if pipeline_code.count("FunctionTransformer(copy)"):
         pipeline_text += """from sklearn.preprocessing import FunctionTransformer
 from copy import copy
 """
+    if not data_file_path:
+        data_file_path = 'PATH/TO/DATA/FILE'
 
     pipeline_text += """
-# NOTE: Make sure that the class is labeled 'target' in the data file
-tpot_data = pd.read_csv('PATH/TO/DATA/FILE', sep='COLUMN_SEPARATOR', dtype=np.float64)
-features = tpot_data.drop('target', axis=1).values
+# NOTE: Make sure that the outcome column is labeled 'target' in the data file
+tpot_data = pd.read_csv('{}', sep='COLUMN_SEPARATOR', dtype=np.float64)
+features = tpot_data.drop('target', axis=1)
 training_features, testing_features, training_target, testing_target = \\
-            train_test_split(features, tpot_data['target'].values, random_state={})
-""".format(random_state)
+            train_test_split(features, tpot_data['target'], random_state={})
+""".format(data_file_path, random_state)
 
     # Add the imputation step if it was used by TPOT
     if impute:
         pipeline_text += """
-imputer = Imputer(strategy="median")
+imputer = SimpleImputer(strategy="median")
 imputer.fit(training_features)
 training_features = imputer.transform(training_features)
 testing_features = imputer.transform(testing_features)
 """
 
     if pipeline_score is not None:
-        pipeline_text += '\n# Average CV score on the training set was:{}'.format(pipeline_score)
+        pipeline_text += '\n# Average CV score on the training set was: {}'.format(pipeline_score)
     pipeline_text += '\n'
 
     # Replace the function calls with their corresponding Python code
@@ -152,7 +165,7 @@ def expr_to_tree(ind, pset):
     return tree
 
 
-def generate_import_code(pipeline, operators, impute=False):
+def generate_import_code(pipeline, operators, impute=False, random_state=None):
     """Generate all library import calls for use in TPOT.export().
 
     Parameters
@@ -163,6 +176,8 @@ def generate_import_code(pipeline, operators, impute=False):
         List of operator class from operator library
     impute : bool
         Whether to impute new values in the feature set.
+    random_state: integer or None
+        Random seed in train_test_split function and exported pipeline.
 
     Returns
     -------
@@ -187,10 +202,6 @@ def generate_import_code(pipeline, operators, impute=False):
     # Build dict of import requirments from list of operators
     import_relations = {op.__name__: op.import_hash for op in operators}
 
-    # Add the imputer if necessary
-    if impute:
-        pipeline_imports['sklearn.preprocessing'] = ['Imputer']
-
     # Build import dict from operators used
     for op in operators_used:
         try:
@@ -204,6 +215,14 @@ def generate_import_code(pipeline, operators, impute=False):
         module_list = ', '.join(sorted(pipeline_imports[key]))
         pipeline_text += 'from {} import {}\n'.format(key, module_list)
 
+    # Add the imputer if necessary
+    if impute:
+        pipeline_text += """from sklearn.impute import SimpleImputer
+"""
+    if random_state is not None:
+        pipeline_text += """from tpot.export_utils import set_param_recursive
+"""
+
     return pipeline_text
 
 
@@ -239,24 +258,38 @@ def _starting_imports(operators, operators_used):
         }
 
 
-def pipeline_code_wrapper(pipeline_code):
+def pipeline_code_wrapper(pipeline_code, random_state=None):
     """Generate code specific to the execution of the sklearn pipeline.
 
     Parameters
     ----------
     pipeline_code: str
         Code that defines the final sklearn pipeline
+    random_state: integer or None
+        Random seed in train_test_split function and exported pipeline.
 
     Returns
     -------
-    Source code for the sklearn pipeline and calls to fit and predict
+    exported_code: str
+        Source code for the sklearn pipeline and calls to fit and predict
 
     """
-    return """exported_pipeline = {}
+    if random_state is None:
+        exported_code = """exported_pipeline = {}
 
 exported_pipeline.fit(training_features, training_target)
 results = exported_pipeline.predict(testing_features)
 """.format(pipeline_code)
+    else:
+        exported_code = """exported_pipeline = {}
+# Fix random state for all the steps in exported pipeline
+set_param_recursive(exported_pipeline.steps, 'random_state', {})
+
+exported_pipeline.fit(training_features, training_target)
+results = exported_pipeline.predict(testing_features)
+""".format(pipeline_code, random_state)
+
+    return exported_code
 
 
 def generate_pipeline_code(pipeline_tree, operators):
@@ -373,3 +406,32 @@ def _combine_dfs(left, right, operators):
 
     return "make_union(\n{},\n{}\n)".\
         format(_indent(_make_branch(left), 4), _indent(_make_branch(right), 4))
+
+
+def set_param_recursive(pipeline_steps, parameter, value):
+    """Recursively iterate through all objects in the pipeline and set a given parameter.
+
+    Parameters
+    ----------
+    pipeline_steps: array-like
+        List of (str, obj) tuples from a scikit-learn pipeline or related object
+    parameter: str
+        The parameter to assign a value for in each pipeline object
+    value: any
+        The value to assign the parameter to in each pipeline object
+    Returns
+    -------
+    None
+
+    """
+    for (_, obj) in pipeline_steps:
+        recursive_attrs = ['steps', 'transformer_list', 'estimators']
+        for attr in recursive_attrs:
+            if hasattr(obj, attr):
+                set_param_recursive(getattr(obj, attr), parameter, value)
+        if hasattr(obj, 'estimator'):  # nested estimator
+            est = getattr(obj, 'estimator')
+            if hasattr(est, parameter):
+                setattr(est, parameter, value)
+        if hasattr(obj, parameter):
+            setattr(obj, parameter, value)
